// Terminal.cpp : Defines the entry point for the application.
//

#include <windows.h>
#include <stdio.h>
#include <conio.h>

#include "Terminal.h"
#include "InfUnltd.h"

// Global Variables:
HINSTANCE hInst;								// current instance
HWND hMainDlg;
HANDLE hThread;
DWORD dwThreadID;
HANDLE hDevice;

bool Opened = false;
bool Enabled = false;
bool OnlyDisplayError = false;
bool AutoRead = true;
//DWORD dwDevice = 0;

#define PROCESS_READ (WM_USER)+1

LRESULT CALLBACK MainDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
int OnDisable(HWND hDlg);
void ASCIIDisplay(HWND hDlg,char output[]);
DWORD DoReceive();
bool GetCheck(HWND handle,UINT item);
void SetCheck(HWND handle,UINT item,bool val);
DWORD WINAPI ReceiverThread();
int StartReceiverThread();
int CloseReceiverThread();
int OnInitDialog(HWND hDlg);
int OnClose(HWND hDlg);
int OnOpen(HWND hDlg);
int OnEnable(HWND hDlg);
int ConvertHexDec(unsigned char * pHex, unsigned char * pDec, unsigned int hexlen);
int OnSend(HWND hDlg);
int OnExit(HWND hDlg);
int OnRST(HWND hDlg);
int OnReset(HWND hDlg);
int OnBytes(HWND hDlg);
int OnFlush(HWND hDlg);

//GUI function
bool GetCheck(HWND handle,UINT item)
{
	if(SendDlgItemMessage(handle,item,BM_GETCHECK,0,0)==TRUE) return true; else return false;
}

//GUI function
void SetCheck(HWND handle,UINT item,bool val)
{
	if(val)SendDlgItemMessage(handle,item,BM_SETCHECK,BST_CHECKED,0);
	else SendDlgItemMessage(handle,item,BM_SETCHECK,BST_UNCHECKED,0);
}

/*
Creates the main dialog window
*/
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;

	hInst = hInstance;

	hMainDlg = CreateDialog(hInstance,MAKEINTRESOURCE(IDD_DIALOG1),HWND_DESKTOP,(DLGPROC)MainDlgProc);
	ShowWindow(hMainDlg,SW_SHOW);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int) msg.wParam;
}

/*
Displays a buffer of text in the log window
*/
void ASCIIDisplay(HWND hDlg,char output[])
{
	SendDlgItemMessage(hDlg,IDC_LOG,EM_REPLACESEL,(WPARAM)0,(LPARAM) (LPCTSTR)output);
}

/*
Displays a buffer of hex data in the log window
*/
void HEXDisplay(HWND hDlg,unsigned char output[],unsigned int len)
{
	char buffer[1024];
	char buffer2[3];

	memset(buffer,0,sizeof(buffer));

	for(unsigned int l = 0; l < len; l++)
	{
		sprintf(buffer2,"%.2X",output[l]);
		strcat(buffer,buffer2);
	}
	
	SendDlgItemMessage(hDlg,IDC_LOG,EM_REPLACESEL,(WPARAM)0,(LPARAM) (LPCTSTR)buffer);
}

/*
Checks how many bytes are ready in the receivebuffer, reads the data and displays them in the log window
*/
DWORD DoReceive()
{
	DWORD BytesReady = 0;
	DWORD dwRead = 0;
	unsigned char readbuffer[2550];
	char buffer[5500];
	SDK_STATUS sdk_status = SDK_SUCCESS;

	//Checks how many bytes are ready in the receivebuffer
	BytesReady = INFUNLTD_Phoenix_BytesInFifo(hDevice);

	//If any bytes ready, read the buffer
	if(BytesReady > 0)
	{
		//If there are more bytes ready than the size of the buffer, read the maximum amount
		if(BytesReady > sizeof(readbuffer)) BytesReady = sizeof(readbuffer);

		//Read the data
		sdk_status = INFUNLTD_Phoenix_Read(hDevice,readbuffer,BytesReady,&dwRead);
		
		//Display error if not successful
		if(sdk_status != SDK_SUCCESS)
		{
			sprintf(buffer,"INFUNLTD_Phoenix_Read(<handle>,<buffer>,%d,<&dwRead>) failed with errorcode %.2X\r\n",BytesReady,sdk_status);
			ASCIIDisplay(hMainDlg,buffer);
			return -1;
		}

		//Display information if detailed information is selected
		sprintf(buffer,"INFUNLTD_Phoenix_Read(<handle>,<buffer>,%d,<&dwRead>) succeeded\r\n",BytesReady);
		if(!OnlyDisplayError) ASCIIDisplay(hMainDlg,buffer);
		
		//Display the data in hex
		sprintf(buffer,"Answer  << ");
		ASCIIDisplay(hMainDlg,buffer);
		HEXDisplay(hMainDlg,readbuffer,dwRead);
		
		//Replace NULL with space
		for(unsigned int l = 0; l < dwRead; l++)
			if(readbuffer[l] == 0) readbuffer[l] = ' ';

		//Display the data in ascii
		readbuffer[dwRead] = 0;
		sprintf(buffer,"\r\nASCII (%d) << %s\r\n",dwRead,readbuffer);
		ASCIIDisplay(hMainDlg,buffer);
	}

	//Return number of bytes read
	return dwRead;
}

/*
Seperate thread that sets the timeout values for communications. It then runs when phoenix mode is enabled.
The thread checks if a PROCESS_READ message has been posted from the GUI, and issues a Read request if so.
The thread also reads the current state of the card switches and informs the user if any cards are removed or arrives.
If Autoread is enabled in the GUI it tries to issue a read request to read any data that might be ready.
*/
DWORD WINAPI ReceiverThread()
{
	SDK_STATUS sdk_status = SDK_SUCCESS;
	DWORD dwCard = 0;
	DWORD dwPrevCard = 0;
	MSG msg;

	char buffer[255];
	
	DWORD dwTimeout = 10000;
	
	//Set timeouts to 10sec for read and write
	sdk_status = INFUNLTD_SetTimeouts(dwTimeout,dwTimeout);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_SetTimeouts(<handle>,%d,%d) failed with errorcode %.2X\r\n",dwTimeout,dwTimeout,sdk_status);
		ASCIIDisplay(hMainDlg,buffer);
		return 1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_SetTimeouts(<handle>,%d,%d) succeeded\r\n",dwTimeout,dwTimeout);
	if(!OnlyDisplayError) ASCIIDisplay(hMainDlg,buffer);

	//Run while phoenix mode is enabled
	while(Enabled)
	{
		//If the message PROCESS_READ has been posted to this thread, receive any data available.
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)!=0)
		if(msg.message == PROCESS_READ) DoReceive();

		//Poll the current card state
		sdk_status = INFUNLTD_GetCardState(hDevice,&dwCard);

		//Display error if not successful
		if(sdk_status != SDK_SUCCESS)
		{
			sprintf(buffer,"INFUNLTD_GetCardState(<handle>,<&dwCard>) failed with errorcode %.2X\r\n",sdk_status);
			ASCIIDisplay(hMainDlg,buffer);
			return 1;
		}

		//If the card state has changed, inform the user
		if(dwCard != dwPrevCard)
		{
			if(dwCard == 0) sprintf(buffer,"Cardswitch state changed to %d (no card inserted)\r\n",dwCard);
			if(dwCard == 1) sprintf(buffer,"Cardswitch state changed to %d (card inserted in normal slot)\r\n",dwCard);
			if(dwCard == 2) sprintf(buffer,"Cardswitch state changed to %d (card inserted in small slot)\r\n",dwCard);
			if(dwCard == 3) sprintf(buffer,"Cardswitch state changed to %d (card inserted in both slots)\r\n",dwCard);

			ASCIIDisplay(hMainDlg,buffer);
		}
		dwPrevCard = dwCard;

		//If autoread is enabled do a read now
		if(AutoRead) DoReceive();

		//Delay until next run
		Sleep(500);
	}

	return 0;
}

/*
Starts the receiver / poll thread
*/
int StartReceiverThread()
{	
	hThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)ReceiverThread,0,0,&dwThreadID);

	return 0;
}

/*
Ends the receiver thread
*/
int CloseReceiverThread()
{
	TerminateThread(hThread,0);

	return 0;
}

/*
This function is called on WM_INITDIALOG, when the dialog window is created
*/
int OnInitDialog(HWND hDlg)
{
	HWND hCombo = 0;
	DWORD dwNumDevices = 0;
	char buffer[25];

	hCombo = GetDlgItem(hDlg,IDC_COMBO_DEVICE);

	//Find out how many devices are connected
	INFUNLTD_GetNumDevices(&dwNumDevices);

	//Fill device combo box
	for(DWORD l = 0; l < dwNumDevices; l++)
	{
		sprintf(buffer,"%d",l);
		SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)buffer);
	}
	SendMessage(hCombo,CB_SETCURSEL,0,0);

	//Fill baudrates combo
	hCombo = GetDlgItem(hDlg,IDC_COMBO_BAUDRATE);
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"2400");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"9600");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"19200");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"28800");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"57600");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"115200");
	SendMessage(hCombo,CB_SETCURSEL,1,0);

	//Fill parity combo
	hCombo = GetDlgItem(hDlg,IDC_COMBO_PARITY);
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"NONE");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"ODD");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"EVEN");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"MARK");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"SPACE");
	SendMessage(hCombo,CB_SETCURSEL,2,0);

	//Fill stopbits combo
	hCombo = GetDlgItem(hDlg,IDC_COMBO_STOPBIT);
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"1");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"2");
	SendMessage(hCombo,CB_SETCURSEL,1,0);

	//Fill voltage combo
	hCombo = GetDlgItem(hDlg,IDC_COMBO_VOLTAGE);
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"5.0v");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"3.3v");
	SendMessage(hCombo,CB_SETCURSEL,0,0);
    
	//Fill frequency combo
	hCombo = GetDlgItem(hDlg,IDC_COMBO_FREQUENCY);
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"3.579 Mhz");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"3.680 Mhz");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"6.000 Mhz");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"8.000 Mhz");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"10.00 Mhz");
	SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)"12.00 Mhz");
	SendMessage(hCombo,CB_SETCURSEL,0,0);

	//Disable Send and Enable buttons on startup
	EnableWindow(GetDlgItem(hDlg,IDC_SEND),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_ENABLE),FALSE);

	//Set checkboxes to default values
	SetCheck(hDlg,IDC_CHECK_ERRORS,!OnlyDisplayError);
	SetCheck(hDlg,IDC_CHECK_AUTO,AutoRead);

	return 0;
}

/*
This function is called when the Close button is pushed.
If phoenix mode is enabled, it disables phoenix mode and then closes the device.
*/
int OnClose(HWND hDlg)
{
	SDK_STATUS sdk_status = SDK_SUCCESS;
	char buffer[255];

	//Invert the button state
	Opened = !Opened;

	//If phoenix mode is enabled, disable it before closing the device
	if(Enabled) OnDisable(hDlg);

	//Close the device
	sdk_status = INFUNLTD_CloseDevice(hDevice);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_CloseDevice(<handle>) failed with errorcode %.2X\r\n",sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_CloseDevice(<handle>) succeeded\r\n");
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Change the Open/Close button state
	SetWindowText(GetDlgItem(hDlg,IDC_OPEN),"Open");
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_DEVICE),TRUE);

	//Disable the Send and Enable buttons
	EnableWindow(GetDlgItem(hDlg,IDC_SEND),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_ENABLE),FALSE);

	return 0;
}

/*
Handles the Open/Close button, if the device is closed it opens the selected device. If the devise is opened it calls OnClose()
*/
int OnOpen(HWND hDlg)
{
	//If the button state is Close, call OnClose()
	if(Opened)
	{
		OnClose(hDlg);
		return 1;
	}

	DWORD dwDevice = 0;
	SDK_STATUS sdk_status = SDK_SUCCESS;
	char buffer[255];
	char buffer2[255];

	//Get the selected device from the list
	dwDevice = (DWORD) SendDlgItemMessage(hDlg,IDC_COMBO_DEVICE,CB_GETCURSEL,0,0);

	//Open the selected device number, the handle hDevice will receive the handle
	sdk_status = INFUNLTD_OpenDeviceFromNum(dwDevice, &hDevice, 0);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_OpenDeviceFromNum(%d) failed with errorcode %.2X\r\n",dwDevice,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_OpenDeviceFromNum(%d) succeeded\r\n",dwDevice);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Invert the button state
	Opened = !Opened;

	SetWindowText(GetDlgItem(hDlg,IDC_OPEN),"Close");
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_DEVICE),FALSE);

	EnableWindow(GetDlgItem(hDlg,IDC_ENABLE),TRUE);

	//Retrieve the SDK version to display in the window
	DWORD dwVersion = INFUNLTD_GetSDKVersion();
	
	//Retrieve the SDK path (path of the DLL) to display in the window
	GetModuleFileName(GetModuleHandle("INFUNLTD"),buffer2,sizeof(buffer2));
	sprintf(buffer,"SDK Version : %d.%.2d\r\n%s",dwVersion/1000,(dwVersion%1000)/10,buffer2);
	SetWindowText(GetDlgItem(hDlg,IDC_VERSION),buffer);

	return 0;
}

/*
This function is called when the Disable phoenix button is pushed. It closes the phoenix interface
and disables the frequency generator.
*/
int OnDisable(HWND hDlg)
{
	//Invert the Enable / Disable button
	Enabled = !Enabled;

	SDK_STATUS sdk_status = SDK_SUCCESS;
	char buffer[255];

	//Disable phoenix mode
	sdk_status = INFUNLTD_Phoenix_Disable(hDevice);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_Disable(<handle>) failed with errorcode %.2X\r\n",sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_Disable(<handle>) succeeded\r\n");
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Disable frequency generator
	DWORD dwFrequency = 0;

	sdk_status = INFUNLTD_Phoenix_SetFrequency(hDevice,dwFrequency);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_SetFrequency(<handle>,%d) failed with errorcode %.2X\r\n",dwFrequency,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_SetFrequency(<handle>,%d) succeeded\r\n",dwFrequency);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Enable buttons that can be used while not in phoenix mode
	SetWindowText(GetDlgItem(hDlg,IDC_ENABLE),"Enable");
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_BAUDRATE),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_PARITY),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_STOPBIT),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_FREQUENCY),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_VOLTAGE),TRUE);
	
	//Disable buttons that cannot be used while not in phoenix mode
	EnableWindow(GetDlgItem(hDlg,IDC_SEND),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_RESET),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_BYTES),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_FLUSH),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_READ),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_CHECK_RST),FALSE);
	
	//Close the receiver thread when not in phoenix mode
	CloseReceiverThread();

	return 0;
}

/*
Handles the Enable / Disable button. If phoenix mode is enabled it calls OnDisable(), otherwise it enabled phoenix mode
and sets the output of the frequency generator.
*/
int OnEnable(HWND hDlg)
{
	//If phoenix mode is enabled, call OnDisable()
	if(Enabled)
	{
		OnDisable(hDlg);
		return 1;
	}

	//Invert the Enable / Disable button
	Enabled = !Enabled;

	SDK_STATUS sdk_status = SDK_SUCCESS;
	char buffer[255];

	//Retrive the baudrate, parity and frequency from the GUI
	unsigned int BaudRate = (unsigned int) SendDlgItemMessage(hDlg,IDC_COMBO_BAUDRATE,CB_GETCURSEL,0,0);
	unsigned int Parity = (unsigned int) SendDlgItemMessage(hDlg,IDC_COMBO_PARITY,CB_GETCURSEL,0,0);
	unsigned int StopBit = (unsigned int) SendDlgItemMessage(hDlg,IDC_COMBO_STOPBIT,CB_GETCURSEL,0,0);
	unsigned int Voltage = (unsigned int) SendDlgItemMessage(hDlg,IDC_COMBO_VOLTAGE,CB_GETCURSEL,0,0);
	unsigned int Frequency = (unsigned int) SendDlgItemMessage(hDlg,IDC_COMBO_FREQUENCY,CB_GETCURSEL,0,0);

	if(StopBit == 0) StopBit = PHOENIX_ONESTOPBIT;
	else StopBit = PHOENIX_TWOSTOPBIT;

	DWORD dwVolt = 0;
	if(Voltage == 0) dwVolt = 5;
	else dwVolt = 3;
    
	unsigned char StopParity = StopBit | Parity;

	//Enable phoenix mode
	sdk_status = INFUNLTD_Phoenix_Enable(hDevice,BaudRate,StopParity);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_Enable(<handle>,%d,%d) failed with errorcode %.2X\r\n",BaudRate,StopBit | Parity,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_Enable(<handle>,%d,%d) succeeded\r\n",BaudRate,StopBit | Parity);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);
/*
	unsigned int baud = 0;
	sdk_status = INFUNLTD_Phoenix_ChangeCustomBaud(hDevice,25000,&baud,StopBit | Parity);
	sprintf(buffer,"INFUNLTD_Phoenix_ChangeCustomBaud(<handle>,%d,%d) succeeded, with baud = %d\r\n",25000,StopBit | Parity,baud);
	ASCIIDisplay(hDlg,buffer);
*/
	//Set frequency generator
	DWORD dwFrequency = 0;
	
	//Convert the combobox index to the corresponding frequency
	if(Frequency == 0) dwFrequency = 3579000;
	if(Frequency == 1) dwFrequency = 3680000;
	if(Frequency == 2) dwFrequency = 6000000;
	if(Frequency == 3) dwFrequency = 8000000;
	if(Frequency == 4) dwFrequency = 10000000;
	if(Frequency == 5) dwFrequency = 12000000;
	
	//Enable and set the frequency generator
	sdk_status = INFUNLTD_Phoenix_SetFrequency(hDevice,dwFrequency);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_SetFrequency(<handle>,%d) failed with errorcode %.2X\r\n",dwFrequency,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_SetVoltage(<handle>,%d) succeeded\r\n",dwVolt);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Set the programming voltage
	sdk_status = INFUNLTD_SetVoltage(hDevice,dwVolt);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_SetVoltage(<handle>,%d) failed with errorcode %.2X\r\n",dwVolt,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_SetVoltage(<handle>,%d) succeeded\r\n",dwVolt);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Set button states, disable buttons that cannot be used while in phoenix mode
	SetWindowText(GetDlgItem(hDlg,IDC_ENABLE),"Disable");
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_BAUDRATE),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_PARITY),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_STOPBIT),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_VOLTAGE),FALSE);
	EnableWindow(GetDlgItem(hDlg,IDC_COMBO_FREQUENCY),FALSE);

	//Set button states, enable buttons that can be used while in phoenix mode
	EnableWindow(GetDlgItem(hDlg,IDC_SEND),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_RESET),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_BYTES),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_FLUSH),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_READ),TRUE);
	EnableWindow(GetDlgItem(hDlg,IDC_CHECK_RST),TRUE);
	
	//Start the receiver thread
	StartReceiverThread();

	return 0;
}

/*
Convert hex values to char
*/
unsigned char HexToChar(unsigned char MSB,unsigned char LSB)
{
	unsigned char Bin = 0;

	if(LSB>=48 && LSB<=57) Bin=LSB-48;
    else if(LSB>= 65 && LSB<=70) Bin=LSB-55;
	else if(LSB>= 65 && LSB<=70) Bin=LSB-55;
	else if(LSB>= 97 && LSB<=102) Bin=LSB-87;

    if(MSB>=48 && MSB<=57) Bin+=(MSB-48)*16;
	else if(MSB>=65 && MSB<=70) Bin+=(MSB-55)*16;
	else if(MSB>= 97 && MSB<=102) Bin+=(MSB-87)*16;

	return Bin;
}

/*
Convert a buffer of hex values to char values
*/
int ConvertHexDec(unsigned char * pHex, unsigned char * pDec, unsigned int hexlen)
{
	unsigned int nospacecounter = 0;

	unsigned char tmp[512];
	unsigned char ch = 0;

	//Remove spaces from command
	for(unsigned int w = 0; w < hexlen; w++)
	{
        if(pHex[w] != ' ') tmp[nospacecounter++] = pHex[w];
	}		

	memset(pDec,0,hexlen/2);

	//Convert command from hex (16bit) to unsigned char (8bit)
	for(unsigned int i = 0; i < (nospacecounter/2); i++)
	{
		ch = HexToChar(tmp[i*2],tmp[(i*2)+1]);
		pDec[i] = ch;
	}

	return nospacecounter/2;
}

/*
Handles the Send button. Data is read in hex from the GUI, converted to binary and Send to the device through phoenix.
*/
int OnSend(HWND hDlg)
{
	SDK_STATUS sdk_status = SDK_SUCCESS;

	char buffer[550];
	unsigned char cmdlinehex[256];
	unsigned char cmdline[512];
	DWORD dwWritten;
	DWORD dwWrite;
	int len;

	//If the device is not opened, display error
	if(!Opened)
	{
		ASCIIDisplay(hDlg,"Open a device first!\r\n");
		return -1;
	}

	//If the phoenix mode is disabled, display error
	if(!Enabled)
	{
		ASCIIDisplay(hDlg,"Enable phoenix for the device first!\r\n");
		return -2;
	}

	//Get the hex data to send
	len = GetWindowText(GetDlgItem(hDlg,IDC_INPUT),(char*)cmdlinehex,sizeof(cmdlinehex));

	//Convert the hex data, to binary data
	dwWrite = ConvertHexDec(cmdlinehex, cmdline,len);

	//sdk_status = INFUNLTD_Phoenix_ResetCard(hDevice);
	//Sleep(1);
	/*
	INFUNLTD_Phoenix_SetRST(hDevice,0);
	Sleep(1);
	INFUNLTD_Phoenix_TrapDelay(hDevice,15,0x0,0);
	
	cmdline[0] = 0x13;
	INFUNLTD_Phoenix_Write(hDevice,cmdline,1,&dwWritten);
	Sleep(10);

	cmdline[0] = 0x13;
	INFUNLTD_Phoenix_Write(hDevice,cmdline,1,&dwWritten);
	Sleep(10);

	cmdline[0] = 0x13;
	INFUNLTD_Phoenix_Write(hDevice,cmdline,1,&dwWritten);
	Sleep(10);

	cmdline[0] = 0x37;
	INFUNLTD_Phoenix_Write(hDevice,cmdline,1,&dwWritten);
	Sleep(10);
*/
	//Send the data to the device, through phoenix mode
	sdk_status = INFUNLTD_Phoenix_Write(hDevice,cmdline,dwWrite,&dwWritten);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_Write(<handle>,<data>,%d,<&dwWritten>) failed with errorcode %.2X\r\n",dwWrite,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_Write(<handle>,<data>,%d,%d) succeeded\r\n",dwWrite,dwWritten);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	//Display the sent data in the log window
	sprintf(buffer,"Command >> %s\r\n",cmdlinehex);
	ASCIIDisplay(hDlg,buffer);

	return 0;
}

/*
Handles Exit of the application, closes the phoenix interface and the device if it is open.
*/
int OnExit(HWND hDlg)
{
	if(Enabled) OnDisable(hDlg);
	if(Opened) OnClose(hDlg);
	EndDialog(hDlg, 0);
	PostQuitMessage(0);

	return 0;
}

/*
Handles the reset checkbox, sets the RST pin of the phoenix interface to the selected state (true/false)
*/
int OnRST(HWND hDlg)
{
	SDK_STATUS sdk_status = SDK_SUCCESS;

	char buffer[255];

	//Gets the checked state of the checkbox
	bool RST = GetCheck(hDlg,IDC_CHECK_RST);
	unsigned int iRST;
	if(RST) iRST = 1;
	else iRST = 0;
	
	//Sets the RST pin high or low
	sdk_status = INFUNLTD_Phoenix_SetRST(hDevice,iRST);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_SetRST(<handle>,%d) failed with errorcode %.2X\r\n",iRST,sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_SetRST(<handle>,%d) succeeded\r\n",iRST);
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	return 0;
}

/*
Handles the reset button, resets the card by calling INFUNLTD_Phoenix_ResetCard
*/
int OnReset(HWND hDlg)
{
	SDK_STATUS sdk_status = SDK_SUCCESS;

	char buffer[255];

	//Reset the card
	sdk_status = INFUNLTD_Phoenix_ResetCard(hDevice);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_ResetCard(<handle>) failed with errorcode %.2X\r\n",sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_ResetCard(<handle>) succeeded\r\n");
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	return 0;
}

/*
Handles the BytesInFifo button, reads the actual amount of data ready to be read
*/
int OnBytes(HWND hDlg)
{
	char buffer[255];

	//Read the amount of bytes ready to be read
	DWORD bytes = INFUNLTD_Phoenix_BytesInFifo(hDevice);

	//Display number of bytes ready
	sprintf(buffer,"INFUNLTD_Phoenix_BytesInFifo(<handle>) = %d\r\n",bytes);
	ASCIIDisplay(hDlg,buffer);

	return 0;
}

/*
Handles the Flush button, flushes / empties the phoenix fifo buffer
*/
int OnFlush(HWND hDlg)
{
	SDK_STATUS sdk_status = SDK_SUCCESS;

	char buffer[255];
	
	//Flush buffer
	sdk_status = INFUNLTD_Phoenix_EmptyFifo(hDevice);

	//Display error if not successful
	if(sdk_status != SDK_SUCCESS)
	{
		sprintf(buffer,"INFUNLTD_Phoenix_EmptyFifo(<handle>) failed with errorcode %.2X\r\n",sdk_status);
		ASCIIDisplay(hDlg,buffer);
		return -1;
	}

	//Display information if detailed information is selected
	sprintf(buffer,"INFUNLTD_Phoenix_EmptyFifo(<handle>) succeeded\r\n");
	if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);

	return 0;
}

/*
Dialog Procedure for the aboutbox
*/
LRESULT CALLBACK AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		OnInitDialog(hDlg);
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
			case IDCANCEL:
			case IDOK:
				EndDialog(hDlg,0);
			break;
		}

		break;
	}
	return FALSE;
}

/*
Main Dialog Procedure for the application
*/
LRESULT CALLBACK MainDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		OnInitDialog(hDlg);
		return TRUE;

	case WM_COMMAND:
		if (HIWORD(wParam)==BN_CLICKED)
		{
			if(LOWORD(wParam)==IDC_CHECK_ERRORS)
			{
				//Gets the checked state of the logging checkbutton
				OnlyDisplayError = !GetCheck(hDlg,IDC_CHECK_ERRORS);
				break;
			}

			if(LOWORD(wParam)==IDC_CHECK_RST)
			{
				//Calls the OnRST function
				OnRST(hDlg);
				break;
			}

			if(LOWORD(wParam)==IDC_CHECK_AUTO)
			{
				//Gets the checked state of the AutoRead checkbutton
				AutoRead = GetCheck(hDlg,IDC_CHECK_AUTO);
				break;
			}
		}

		//Dispatch the functions matching the pushed buttons
		switch(LOWORD(wParam))
		{
			case IDCANCEL:
			case IDOK:
				OnExit(hDlg);
			break;

			case IDC_RESET:
				OnReset(hDlg);
			break;

			case IDC_READ:
				//Posts a PROCESS_READ command to the receive thread, this way all reading is done in the same thread
				PostThreadMessage(dwThreadID,PROCESS_READ,0,0);
			break;

			case IDC_BYTES:
				OnBytes(hDlg);
			break;

			case IDC_FLUSH:
				OnFlush(hDlg);
			break;

			case IDC_OPEN:
				OnOpen(hDlg);
			break;

			case IDC_ENABLE:
				OnEnable(hDlg);
			break;

			case IDC_SEND:
				OnSend(hDlg);
			break;

			case IDC_CLEAR:
				SetWindowText(GetDlgItem(hDlg,IDC_LOG),"");
			break;

			case IDC_ABOUT:
				DialogBox(hInst,MAKEINTRESOURCE(IDD_ABOUTBOX),hDlg,(DLGPROC)AboutDlgProc);
			break;

			case IDC_BUTTON5:
				{
					char buffer[255];
					int status = INFUNLTD_SetVoltage(hDevice,5);
					sprintf(buffer,"INFUNLTD_Phoenix_SetVoltage(<handle>,%d) succeeded\r\n",5);
					if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);
				}
			break;

			case IDC_BUTTON0:
				{
					char buffer[255];
					int status = INFUNLTD_SetVoltage(hDevice,0);
					sprintf(buffer,"INFUNLTD_Phoenix_SetVoltage(<handle>,%d) succeeded\r\n",0);
					if(!OnlyDisplayError) ASCIIDisplay(hDlg,buffer);
				}
			break;
		}

		break;
	}
	return FALSE;
}
