#include <windows.h>
#include "HexToBin.h"

HexToBin::HexToBin(void)
{
	BinData = new unsigned char[ARRAY_SIZE];

	if(!BinData) return;

	memset(BinData,0,ARRAY_SIZE);
	memset(BitData,0,ARRAY_SIZE);

	Size=0;
	wFuses=0;
	LowAdr=HighAdr=0;

	CRD = false;
}

HexToBin::~HexToBin(void)
{
	if(BinData) delete [] BinData;
}

unsigned int HexToBin::DataSize()
{
	return Size;
}

void HexToBin::ClearBuffers(unsigned char val)
{
	LowAdr=0;
	HighAdr=0;
	Size=0;
	wFuses=0;

	memset(BinData,val,ARRAY_SIZE);
	memset(BitData,0,ARRAY_SIZE);
}

bool HexToBin::PrepareForBinInput(unsigned int Low, unsigned int High)
{
	this->LowAdr=Low;
	this->HighAdr=High;
	Size=0;

	return true;
}

unsigned char HexToBin::OutputBin(unsigned int Index)
{
	return BinData[Index];
}

int HexToBin::InputBin(unsigned char data)
{
	if(Size < ARRAY_SIZE)
	{
		BitData[Size]=true;
		BinData[Size++]=data;

		HighAdr = Size-1;
	}

	return 0;
}

int HexToBin::ParseBin(char szFile[],int iType)
{
	FILE *f;

	BIN=true;
	HEX=false;

	memset(BinData,0xFF,ARRAY_SIZE);
	memset(BitData,0,ARRAY_SIZE);

	Size=-1;
	if((f=fopen(szFile,"rb"))==NULL) return 1;
	while(!feof(f))
	{
		Size++;
		if(Size>=ARRAY_SIZE) break;
		BinData[Size]=(unsigned char)fgetc(f);
		BitData[Size]=true;
	}
	HighAdr=Size-1;

	BitData[Size]=false;

	LowAdr=0;
	fclose(f);
	return 0;
}

int HexToBin::ParseHex(char szFile[],int iType)
{
	FILE *f;
	char LineBuffer[255];
	
	unsigned int CheckSum,CheckSumResult;
	unsigned short int NumOfDataBytes=0;
	unsigned short int RecordType=0;
	unsigned int i;
	bool CheckSumError = false;

	unsigned int Adr=0;
	unsigned int MSBAdr=0;

	BIN=false;
	HEX=true;
	CRD=false;

	bool intel8=true;
	const unsigned char HexLineOverhead = 11;
	unsigned int StrLen = 0;

	memset(BinData,0xFF,ARRAY_SIZE);
	memset(BitData,0,ARRAY_SIZE);

	LowAdr=ARRAY_SIZE;
	HighAdr=0;
	Size=0;
	f=fopen(szFile,"r");
	if(f==NULL) return 1;

	while(fgets(LineBuffer,sizeof(LineBuffer),f)!=NULL)
	{
		if(LineBuffer[0]!=':') 
		{
			if(LineBuffer[0]=='\n') continue; //Support for empty lines
			if(LineBuffer[0]==';') continue; //Support for ; commented lines
			return 2;
		}else
		{
			RecordType = 1;
			if(strcmp(LineBuffer,":00000001FF")==0) break;
		}
		
		CheckSum=0;
		for(i=1;i<strlen(LineBuffer)-3;i+=2) CheckSum+=HexToChar(LineBuffer[i],LineBuffer[i+1]);
		CheckSum=256-(CheckSum%256);
		if(CheckSum==256) CheckSum=0;

		NumOfDataBytes=HexToChar(LineBuffer[1],LineBuffer[2]);

		Adr=HexToChar(LineBuffer[5],LineBuffer[6])+(HexToChar(LineBuffer[3],LineBuffer[4])*256);

		RecordType=HexToChar(LineBuffer[7],LineBuffer[8]);
		if(RecordType==4)
			MSBAdr = GetExtendedAddress(LineBuffer);
		else 
		{
			CheckSumResult=HexToChar(LineBuffer[strlen(LineBuffer)-3],LineBuffer[strlen(LineBuffer)-2]);

			StrLen = strlen(LineBuffer);
			if((StrLen-1) == (NumOfDataBytes*4 + HexLineOverhead)) intel8 = false;
			
			if(intel8)
			for(i=0;i<NumOfDataBytes;i++) 
			{
				unsigned int TotalAdr = (MSBAdr<<16)|(Adr+i);
				
				if(TotalAdr>=ARRAY_SIZE) return 6;
				BinData[TotalAdr]=HexToChar(LineBuffer[9+i*2],LineBuffer[10+i*2]);
				BitData[TotalAdr]=1;
				if(TotalAdr>HighAdr) HighAdr=TotalAdr;
				if(TotalAdr<LowAdr) LowAdr=TotalAdr;
			}else
			for((unsigned int)i=0;i<(unsigned int)NumOfDataBytes*2;i+=2) 
			{
				unsigned int TotalAdr = (MSBAdr<<16)|((Adr*2)+i);
				if(TotalAdr>=ARRAY_SIZE) return 6;

				BinData[TotalAdr+1]=HexToChar(LineBuffer[9+i*2],LineBuffer[10+i*2]);
				BinData[TotalAdr]=HexToChar(LineBuffer[11+i*2],LineBuffer[12+i*2]);
				
				BitData[TotalAdr]=1;
				BitData[TotalAdr+1]=1;

				if(TotalAdr>HighAdr) HighAdr=TotalAdr;
				if(TotalAdr<LowAdr) LowAdr=TotalAdr;
			}

			if(CheckSum!=CheckSumResult) 
				CheckSumError=true;
			Size+=NumOfDataBytes;
		}
	}
	fclose(f);

	if(CheckSumError) return 4;
	if(RecordType==1) return 0;	

	return 5;
}

WORD HexToBin::GetExtendedAddress(char str[])
{
	unsigned char msb = HexToChar(str[9],str[10]);
	unsigned char lsb = HexToChar(str[11],str[12]);

	return MAKEWORD(lsb,msb);
}

int HexToBin::SaveBin(char szFile[])
{
	DWORD dwWritten;

	HANDLE hFile = CreateFile(szFile,GENERIC_WRITE,0,0,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,SECURITY_ANONYMOUS);
	if(hFile==NULL) return -1;
	if(!WriteFile(hFile,BinData,HighAdr+1,&dwWritten,NULL)) return -2;
	CloseHandle(hFile);
	if(dwWritten!=HighAdr+1) return -3;
	return 0;
}

unsigned char HexToBin::HexToChar(unsigned char MSB,unsigned char LSB)
{
	unsigned char Bin=0;

	if(LSB>=48 && LSB<=57) Bin=LSB-48;
    else if(LSB>= 65 && LSB<=70) Bin=LSB-55;
	else if(LSB>= 65 && LSB<=70) Bin=LSB-55;
	else if(LSB>= 97 && LSB<=102) Bin=LSB-87;

    if(MSB>=48 && MSB<=57) Bin+=(MSB-48)*16;
	else if(MSB>=65 && MSB<=70) Bin+=(MSB-55)*16;
	else if(MSB>= 97 && MSB<=102) Bin+=(MSB-87)*16;

	return Bin;
}

HexToBin& HexToBin::operator=(const HexToBin& src)
{
	this->wFuses=src.wFuses;
	this->LowAdr=src.LowAdr;
	this->HighAdr=src.HighAdr;
	this->BIN=src.BIN;
	this->HEX=src.HEX;
	this->Size=src.Size;

	memcpy(this->BitData,src.BitData,ARRAY_SIZE);
	memcpy(this->BinData,src.BinData,ARRAY_SIZE);

	return *this;
}
