// LED.cpp : Defines the entry point for the application.
//

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>

#include "resource.h"
#include "infunltd.h"

/*
Fill the device list with the number of devices found
*/
int OnInitDialog(HWND hDlg)
{
	HWND hCombo = 0;
	DWORD dwNumDevices = 0;
	char buffer[25];

	hCombo = GetDlgItem(hDlg,IDC_COMBO1);

	//Find out how many devices are connected
	INFUNLTD_GetNumDevices(&dwNumDevices);

	for(DWORD l = 0; l < dwNumDevices; l++)
	{
		sprintf(buffer,"%d",l);
		SendMessage(hCombo,CB_ADDSTRING,0,(LPARAM)buffer);
	}
	
	SendMessage(hCombo,CB_SETCURSEL,0,0);

	SetWindowText(GetDlgItem(hDlg,IDC_EDIT1),"100");
	SetWindowText(GetDlgItem(hDlg,IDC_EDIT2),"100");
	SetWindowText(GetDlgItem(hDlg,IDC_EDIT3),"100");
	SetWindowText(GetDlgItem(hDlg,IDC_EDIT4),"255");

	return 0;
}

/*
Converts a 0-100% value into a 16bit value
*/
unsigned int ConvertPercentTo16bit(unsigned char percent)
{
	const float a = 40;
	const float b = (float)1.07;

	unsigned short int duty = (unsigned short int)(a * pow(b,percent));

	return duty;
}

/*
Sets the LED with the values specified in the GUI
*/
int OnSet(HWND hDlg)
{
	HWND hCombo = 0;
	DWORD dwDevice = 0;
	HANDLE hDevice;
	unsigned int uiR = 0;
	unsigned int uiG = 0;
	unsigned int uiB = 0;
	unsigned char ucFrq = 0;
	char buffer[10];

	SDK_STATUS sdk_status = SDK_SUCCESS;

	//Read and convert the READ value
	GetWindowText(GetDlgItem(hDlg,IDC_EDIT1),buffer,6);
	uiR = atoi(buffer);
	if(uiR < 0 || uiR > 100)
	{
		MessageBox(hDlg,"Red value should be between 0-100%","Error",MB_OK);
		return -2;
	}
	uiR = ConvertPercentTo16bit(uiR);
	
	//Read and convert the GREEN value
	GetWindowText(GetDlgItem(hDlg,IDC_EDIT2),buffer,6);
	uiG = atoi(buffer);
	if(uiG < 0 || uiG > 100)
	{
		MessageBox(hDlg,"Green value should be between 0-100%","Error",MB_OK);
		return -2;
	}
	uiG = ConvertPercentTo16bit(uiG);

	//Read and convert the BLUE value
	GetWindowText(GetDlgItem(hDlg,IDC_EDIT3),buffer,6);
	uiB = atoi(buffer);
	if(uiB < 0 || uiB > 100)
	{
		MessageBox(hDlg,"Blue value should be between 0-100%","Error",MB_OK);
		return -2;
	}
	uiB = ConvertPercentTo16bit(uiB);
	
	//Read the Frequency value
	GetWindowText(GetDlgItem(hDlg,IDC_EDIT4),buffer,6);
	ucFrq = atoi(buffer);
	if(atoi(buffer) < 0 || atoi(buffer) > 255)
	{
		MessageBox(hDlg,"PWM value should be between 0-255","Error",MB_OK);
		return -2;
	}
	
	
	hCombo = GetDlgItem(hDlg,IDC_COMBO1);
	dwDevice = (DWORD) SendMessage(hCombo,CB_GETCURSEL,0,0);
	
	//Open the selected device
	sdk_status = INFUNLTD_OpenDeviceFromNum(dwDevice,&hDevice,0);

	//Report failure if any
	if(sdk_status != SDK_SUCCESS)
	{
		MessageBox(hDlg,"Error opening device","Error",MB_OK);
		return -1;
	}
	
	//Set the LED to the selected values
	sdk_status = INFUNLTD_SetLEDState(hDevice,uiR,uiG,uiB,ucFrq);
	if(sdk_status != SDK_SUCCESS)
	{
		MessageBox(hDlg,"Error setting LED on device","Error",MB_OK);
		return -1;
	}

	//Close the opened device
	sdk_status = INFUNLTD_CloseDevice(hDevice);
	if(sdk_status != SDK_SUCCESS)
	{
		MessageBox(hDlg,"Error closing device","Error",MB_OK);
		return -1;
	}

	return 0;
}

BOOL CALLBACK DlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
            OnInitDialog(hDlg);
		break;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg,0);
				return TRUE;
			}

			if (LOWORD(wParam) == IDOK) 
			{
				OnSet(hDlg);
			}
			break;
	}
    return FALSE;
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG1),HWND_DESKTOP,(DLGPROC)DlgProc);

	return 0;
}



