#pragma once

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the INFUNLTD_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// INFUNLTD_SDK functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef INFUNLTD_EXPORTS
#define INFUNLTD_SDK __declspec(dllexport)
#else
#define INFUNLTD_SDK __declspec(dllimport)
#endif

enum phoenix_parity
{
	PHOENIX_PARITY_NONE, 
	PHOENIX_PARITY_ODD,
	PHOENIX_PARITY_EVEN,
	PHOENIX_PARITY_MARK,
	PHOENIX_PARITY_SPACE
};

enum phoenix_stopbits
{
	PHOENIX_TWOSTOPBIT = 0x00, 
	PHOENIX_ONESTOPBIT = 0x20, 
};

enum phoenix_baud
{
	PHOENIX_BAUD_2400, 
	PHOENIX_BAUD_9600, 
	PHOENIX_BAUD_19200, 
	PHOENIX_BAUD_28800, 
	PHOENIX_BAUD_38400, 
	PHOENIX_BAUD_57600, 
	PHOENIX_BAUD_115200
};

enum phoenix_frq
{
	PHOENIX_FRQ_3579000 = 3579000, 
	PHOENIX_FRQ_3680000 = 3680000, 
	PHOENIX_FRQ_6000000 = 6000000
};

enum cardstates
{
	CARD_NONE, 
	CARD_SMARTCARD, 
	CARD_SIMCARD,
	CARD_BOTH
};

typedef	int	SDK_STATUS;

#define		SDK_SUCCESS					0x00
#define		SDK_INVALID_HANDLE			0x01
#define		SDK_DEVICE_NOT_FOUND		0xFF
#define		SDK_READ_ERROR				0x02
#define		SDK_RX_QUEUE_NOT_READY		0x03
#define		SDK_WRITE_ERROR				0x04
#define		SDK_INVALID_PARAMETER		0x06
#define		SDK_INVALID_REQUEST_LENGTH	0x07
#define		SDK_DEVICE_IO_FAILED		0x08
#define		SDK_PHOENIX_NOT_ENABLED		0x60
#define		SDK_ALREADY_OPENED			0x09

#ifdef __cplusplus
extern "C" {
#endif

//General
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_GetNumDevices(LPDWORD lpdwNumDevices);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_OpenDeviceFromNum(DWORD dwDevice, HANDLE* hDevice, LPVOID lpDeviceString);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_OpenDeviceFromDeviceString(DWORD* pdwDevice,HANDLE* hDevice, LPCVOID lpDeviceString);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_GetDeviceNumFromDeviceString(DWORD* pdwDevice, LPCVOID lpDeviceString);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_GetDeviceStringFromDeviceNum(DWORD dwDevice, LPVOID lpDeviceString);

INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_CloseDevice(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_SetLEDState(HANDLE hDevice, unsigned int R, unsigned int G,unsigned int B, unsigned char PWMFrq);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_GetCardState(HANDLE hDevice, LPDWORD lpdwCardState);
INFUNLTD_SDK	DWORD			_stdcall INFUNLTD_GetFirmwareVersion(HANDLE hDevice);
INFUNLTD_SDK	DWORD			_stdcall INFUNLTD_GetSDKVersion();
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_SetVoltage(HANDLE hDevice, DWORD dwVolt);

//Communications
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Write(HANDLE hDevice, LPCVOID lpBuffer,DWORD nNumberOfBytesToWrite, LPDWORD lpdwBytesWritten);
INFUNLTD_SDK 	SDK_STATUS		_stdcall INFUNLTD_Read(HANDLE hDevice, LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpdwBytesRead);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_SetTimeouts(DWORD dwRead, DWORD dwWrite);
INFUNLTD_SDK	DWORD			_stdcall INFUNLTD_BytesInFifo(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_EmptyFifo(HANDLE hDevice);

//Phoenix
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Write(HANDLE hDevice, LPCVOID lpBuffer,DWORD nNumberOfBytesToWrite, LPDWORD lpdwBytesWritten);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Read(HANDLE hDevice, LPVOID lpBuffer,DWORD nNumberOfBytesToRead, LPDWORD lpdwBytesRead);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Enable(HANDLE hDevice, unsigned int baud, unsigned int stopbitparity);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Change(HANDLE hDevice, unsigned int baud, unsigned int stopbitparity);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_ChangeCustomBaud(HANDLE hDevice, unsigned int baud, unsigned int * ActualBaud, unsigned int stopbitparity);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_SetFrequency(HANDLE hDevice, DWORD dwFrq);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Disable(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_SetRST(HANDLE hDevice, unsigned int state);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_ResetCard(HANDLE hDevice);
INFUNLTD_SDK	DWORD			_stdcall INFUNLTD_Phoenix_BytesInFifo(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_EmptyFifo(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Trap(HANDLE hDevice, unsigned char Delay, unsigned char Trapvalue);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_TrapDelay(HANDLE hDevice, unsigned char Delay, unsigned char Trapvalue, DWORD dwSleep);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_BreakTrap(HANDLE hDevice, unsigned char Delay, unsigned char Trapvalue);

INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_ReEnable(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_Enabled(HANDLE hDevice);
//INFUNLTD_SDK	SDK_STATUS		_stdcall INFUNLTD_Phoenix_GetLastSettings(int * SavedBaud, int * SavedParity);

//Multi application support
INFUNLTD_SDK	DWORD		_stdcall INFUNLTD_MultiApp_EnterExclusiveAccess(int iDeviceNo, int iLockNo, DWORD dwTimeout);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_MultiApp_ExitExclusiveAccess(int iDeviceNo, int iLockNo);

INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SetInfo(char *pUsr, char *pPws, bool pUsingGina);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_GetInfo(char *pUsr, char *pPws, bool *pUsingGina);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_AlreadyOpened(DWORD dwDevice);

//SLE
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousBegin(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousEnd(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousProcess(HANDLE hDevice, unsigned char iostate);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousWriteCommand(HANDLE hDevice, unsigned char control, unsigned char adr, unsigned char data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousWriteOneByteLSB(HANDLE hDevice, unsigned char data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousWriteOneByteMSB(HANDLE hDevice, unsigned char data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousCycleClock(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousRead8bit(HANDLE hDevice,unsigned char count, unsigned char* data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousReadXbytesYbits(HANDLE hDevice,unsigned char bytecount, unsigned char bitcount, unsigned char* data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_SynchronousSetRST(HANDLE hDevice, unsigned char state);

//i2c
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_IICClockOutStart(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_IICClockOutStop(HANDLE hDevice);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_IICClockOutByte(HANDLE hDevice, unsigned char data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_IICClockInByteAck(HANDLE hDevice, unsigned char *data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_IICClockInByteNack(HANDLE hDevice, unsigned char *data);
INFUNLTD_SDK	SDK_STATUS	_stdcall INFUNLTD_IICCycleClock(HANDLE hDevice);

#ifdef __cplusplus
}
#endif
